"""Models for BSB-Lan."""

import json
from types import SimpleNamespace as Namespace

import attr


class State:
    """Convert Data to valid keys and convert to object attributes."""

    # list of parameter needed for climate device
    heating_circuit1 = [
        700,  # hvac_mode
        710,  # target_temperature
        711,  # target_temperature_high
        712,  # target_temperature_low
        714,  # min_temp
        730,  # max_temp
        900,  # hvac_action?
        8000,  # status_heating_circuit1
        8740,  # current_temperature room1
        8749,  # Raumthermostat1
    ]

    heating_circuit2 = [
        1000,
        1010,
        1011,
        1012,
        1014,
        1030,
        1200,
        8001,  # status_heating_circuit2
        8770,
    ]

    @staticmethod
    def from_dict(data: dict):
        """Return State object from BSBLan API response."""

        # need the states fom homeassistant
        KEYS_TO_STATE = {
            "700": "hvac_mode",
            "710": "target_temperature",
            # "902": "target_temperature_high",
            "711": "target_temperature_high",  # comfort max temp
            "712": "target_temperature_low",
            "714": "min_temp",  # frost_protection_temp
            "730": "max_temp",  # changeover_temperature
            "900": "hvac_action",
            "8000": "status_heatingcircuit1",
            "8740": "current_temperature",
        }
        RETURN_CIRCUIT1_DICT = {}
        # only retrieve keys with valid values
        for k in KEYS_TO_STATE.keys() & data.keys():
            RETURN_CIRCUIT1_DICT[k] = data[k]
        # print(f"new_dict: {new_dict}")
        RETURN_CIRCUIT1_DICT = {
            KEYS_TO_STATE.get(k, k): v for k, v in RETURN_CIRCUIT1_DICT.items()
        }

        return json.loads(
            json.dumps(RETURN_CIRCUIT1_DICT), object_hook=lambda d: Namespace(**d)
        )


@attr.s(auto_attribs=True, frozen=True)
class Info:
    """Object holding the heatingsystem info."""

    controller_family: float
    controller_variant: float
    device_identification: str

    @staticmethod
    def from_dict(data: dict):
        """Return Info object from BSBLan API response."""
        # print(f"device {data}")

        return Info(
            controller_family=data["6225"]["value"],
            controller_variant=data["6226"]["value"],
            device_identification=data["6224"]["value"],
        )



"""Asynchronous Python client for BSB-Lan."""
import asyncio
import logging
import socket
from typing import Any, Mapping, Optional

import aiohttp
import async_timeout
from yarl import URL

from .__version__ import __version__
from .exceptions import BSBLanConnectionError, BSBLanError
from .models import Info, State

logging.basicConfig(level=logging.DEBUG)


class BSBLan:
    """Main class for handling connections with BSBLan."""

    def __init__(
        self,
        host: str,
        port: int = 80,
        request_timeout: int = 10,
        session: aiohttp.client.ClientSession = None,
        username: str = None,
        password: str = None,
        passkey: str = None,
    ) -> None:
        """Initialize connection with BSBLan."""
        self._session = session
        self._close_session = False
        self.host = host
        self.port = port
        self.request_timeout = request_timeout
        self.username = username
        self.password = password
        self.passkey = passkey
        self._heatingcircuit1: list = []

    async def _request(
        self,
        method: str = "POST",
        data: Optional[dict] = None,
        params: Optional[Mapping[str, str]] = None,
    ) -> Any:
        """Handle a request to a BSBLan device."""

        base_path = "/JQ" if data is None else "/JS"
        if self.passkey is not None:
            base_path = f"/{self.passkey}{base_path}"

        url = URL.build(
            scheme="http", host=self.host, port=self.port, path=base_path
        ).join(URL())

        auth = None
        if self.username and self.password:
            auth = aiohttp.BasicAuth(self.username, self.password)

        headers = {
            "User-Agent": f"PythonBSBLan/{__version__}",
            "Accept": "application/json, */*",
        }

        if self._session is None:
            self._session = aiohttp.ClientSession()
            self._close_session = True

        try:
            with async_timeout.timeout(self.request_timeout):
                response = await self._session.request(
                    method,
                    url,
                    auth=auth,
                    params=params,
                    json=data,
                    headers=headers,
                )
                response.raise_for_status()
        except asyncio.TimeoutError as exception:
            raise BSBLanConnectionError(
                "Timeout occurred while connecting to BSBLan device."
            ) from exception
        except (
            aiohttp.ClientError,
            aiohttp.ClientResponseError,
            socket.gaierror,
        ) as exception:
            raise BSBLanConnectionError(
                "Error occurred while communicating with BSBLan device."
            ) from exception

        content_type = response.headers.get("Content-Type", "")
        if "application/json" not in content_type:
            text = await response.text()
            raise BSBLanError(
                "Unexpected response from the BSBLan device",
                {"Content-Type": content_type, "response": text},
            )
        return await response.json()

    async def _scan(self, parameters) -> list:
        """Scan for parameters that return a value.

        input: list to scan

        output: string for scanning the params

        """
        # We should add parameters here using scan function.
        # By default we need a list with basic params.

        # convert list to string
        parameters = ",".join(str(e) for e in parameters)
        data = await self._request(params={"Parameter": f"{parameters}"})
        logging.debug("data: %s", data)
        notValidData = []
        notValidData2 = []
        for k, v in data.items():
            if not v.get("value"):
                # print(f"Invalid: {k}")
                notValidData.append(k)
            if v.get("value") == "---":
                notValidData2.append(k)

        # remove parameters with no returning value
        # print(f"DataNotValid: {notValidData} and {notValidData2}")
        for i in notValidData or notValidData2:
            data.pop(i)

        # join parameters to create one string
        parameters = []
        for i in data.keys():
            parameters.append(i)
        parameters = ",".join(parameters)
        logging.debug("params: %s", parameters)

        return parameters

    async def state(self) -> State:
        """Get the current state from BSBLan device."""

        if not self._heatingcircuit1:
            logging.info("scanning for state Parameters")
            parameters = State.heating_circuit1

            self._heatingcircuit1 = await self._scan(parameters)
        logging.info("get state heatingcircuit1")
        data = await self._request(params={"Parameter": f"{self._heatingcircuit1}"})

        return State.from_dict(data)

    async def info(self) -> Info:
        """Get information about the current heating system config."""
        data = await self._request(params={"Parameter": "6224,6225,6226"})
        return Info.from_dict(data)

    async def thermostat(
        self,
        target_temperature: Optional[str] = None,
        hvac_mode: Optional[str] = None,
    ) -> None:
        """Change the state of the thermostat through BSB-Lan."""

        state = {}

        if target_temperature is not None:
            state["Parameter"] = "710"
            state["Value"] = target_temperature
            state["Type"] = "1"
        if hvac_mode is not None:
            state["Parameter"] = "700"
            state["EnumValue"] = hvac_mode
            state["Type"] = "1"
        # Type needs to be 1 to really set value.
        # Now it only checks if it could set value.
        await self._request("", data=state)
        # return Thermostat.from_dict(data)

    async def close(self) -> None:
        """Close open client session."""
        if self._session and self._close_session:
            await self._session.close()

    async def __aenter__(self) -> "BSBLan":
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.close()



"""Exceptions for for BSB-Lan."""


class BSBLanError(Exception):
    """Generic BSBLan exception."""

    pass


class BSBLanConnectionError(BSBLanError):
    """BSBLan connection exception."""

    pass
